@@ -39,6 +39,9 @@ import json
 import sys
 import socket
 import time
+import ipaddress
+from datetime import datetime, timedelta
+from dateutil import parser
 from urllib.parse import urljoin, urlparse, urlunparse
 from requests.auth import HTTPBasicAuth, AuthBase
 from .ServiceRoot import RfServiceRoot
@@ -57,8 +60,8 @@ class RfTransport():
     def __init__(self):
         # constant parameters-- these dont change and are not updated
         self.program="redfishtool"              # program name (in case we want to change it)
-        self.version="1.0.6"                    # this redfishtool version
-        self.releaseDate="10/12/2018"           # release date for this version of redfishtool
+        self.version="1.1.3"                    # this redfishtool version
+        self.releaseDate="05/01/2020"           # release date for this version of redfishtool
         self.downloadFrom="https://github.com/DMTF/Redfishtool" # where to find redfishtool
         self.magic="12345"                      # used for debug to test for a known parameter in this object
         self.UNAUTHENTICATED_API=1              # unauthenticated API that doesn't send credentials in body data
@@ -87,6 +90,7 @@ class RfTransport():
         self.auth="Basic"  # or "Session" using Basic as default now
         self.timeout=10         # http transport timeout in seconds, stored as int here
         self.checkProtocolVer=False  # if -C option, then we need to check/verify the protocol ver. dflt=false
+        self.blocking=True
 
         # more option parsing variables
         self.prop=None
@@ -212,7 +216,15 @@ class RfTransport():
 
             # calculate the rootUri including scheme,rhost,rootPath properly
             scheme=rft.getApiScheme(rft.UNAUTHENTICATED_API)
-            scheme_tuple=[scheme,rft.rhost, rft.rootPath, "","",""]
+
+            rhost = rft.rhost
+            try:
+                if ipaddress.ip_address(rhost).version == 6:
+                    rhost = '[{}]'.format(rhost)
+            except ValueError:
+                pass
+
+            scheme_tuple=[scheme, rhost, rft.rootPath, "","",""]
             rootUrl=urlunparse(scheme_tuple)
             rft.rootUri=rootUrl
             # save parameters
@@ -532,6 +544,15 @@ class RfTransport():
                 elif( (r.status_code==200) and (method=="HEAD") ):
                     success=True
                     return(rc,r,False,None)
+                elif (r.status_code == 202 and method in ["DELETE", "PATCH", "POST", "PUT"]):
+                    success = True
+                    if rft.blocking and r.headers.get("Location"):
+                        return rft.waitForTask(r, urlBase2, headers=hdrs, auth=authType,
+                                               verify=verify, jsonData=jsonData, **kwargs)
+                    else:
+                        rft.printTaskStatus("Task Monitor URL is %s\n" %
+                                            r.headers.get("Location", "<not available>"))
+                        return (rc, r, False, None)
                 elif((r.status_code==200) or (r.status_code==201) ):  
                     if( jsonData is True):
                         try:
@@ -572,14 +593,93 @@ class RfTransport():
                         return rc, r, jsonData, respd
                 elif( r.status_code!=200):
                     success=False
-                    rft.printErr("Transport: processing response status codes")
+                    rft.printErr("Transport: unexpected response status code {} for method {}"
+                                 .format(r.status_code, method))
                     return(5,r,False,None)
 
         # Should not get here, but log error if we do
         rft.printErr("Transport: Internal error; reached end of function without returning")
         return 5, r, False, None
 
+    def sleepFor(self, response):
+        retry_after = response.headers.get("Retry-After", 1)
+        if isinstance(retry_after, int) or retry_after.isdigit():
+            # Retry-After: 120
+            sleep_for = timedelta(seconds=int(retry_after))
+        else:
+            # Retry-After: Fri, 31 Dec 1999 23:59:59 GMT
+            sleep_for = parser.parse(retry_after) - datetime.now()
+        return max(0, sleep_for.total_seconds())
 
+    def taskStatus(self, response):
+        task_state = "Running"
+        if "application/json" in response.headers.get("Content-Type", ""):
+            try:
+                d = response.json()
+                task_state = d.get("TaskState", "Running")
+                task_percent = d.get("PercentComplete")
+                if task_percent is not None:
+                    return "Task is {}: {}% complete\r".format(
+                        task_state, task_percent)
+            except Exception:
+                pass
+        return "Task is {}\r".format(task_state)
+
+    def printTaskStatus(self, msg):
+        """Print status to STDERR so that scripts that are expecting only JSON
+        on STDOUT do not break. Do not display task status if STDERR is
+        redirected.
+        """
+        if sys.stderr.isatty():
+            sys.stderr.write("\x1b[2K")
+            sys.stderr.write(msg)
+            sys.stderr.flush()
+
+    def waitForTask(self, r, url_base, headers=None, auth=None,
+                    verify=False, max_wait=60, jsonData=True, **kwargs):
+        self.printVerbose(2, "Transport:waitForTask: enter")
+        if headers is None:
+            headers = {}
+        else:
+            headers = headers.copy()
+        if "Content-Type" in headers:
+            del headers["Content-Type"]
+        timeout = (self.waitTime, self.timeout)
+        location = r.headers.get("Location")
+        uri = urlparse(location).path
+        url = urljoin(url_base, uri)
+        timeout_at = time.time() + max_wait  # time to give up on async task
+        while r.status_code == 202:
+            self.printTaskStatus(self.taskStatus(r))
+            sleep_for = self.sleepFor(r)
+            self.printVerbose(2, "Transport:waitForTask: sleep for %s seconds" % sleep_for)
+            time.sleep(sleep_for)
+            self.printVerbose(3, "Transport:SendRecv:    {} {}".format('GET', url))
+            t1 = time.time()
+            r = requests.get(url, headers=headers, auth=auth, verify=verify,
+                             timeout=timeout, **kwargs)
+            self.elapsed = time.time() - t1
+            self.printStatus(1, r=r)
+            self.printStatus(2, r=r)
+            if time.time() >= timeout_at and r.status_code == 202:
+                break
+        if r.status_code == 202:
+            self.printTaskStatus("Task still Running (max wait time exceeded)\n")
+        else:
+            self.printTaskStatus("Task is Done\n")
+        if r.ok:
+            if jsonData and "application/json" in r.headers.get("Content-Type", ""):
+                try:
+                    d = r.json()
+                    return 0, r, jsonData, d
+                except Exception as e:
+                    self.printErr("Caught exception decoding JSON from async "
+                                  "task response; Exception: %s" % e)
+                    return 5, r, False, None
+            else:
+                return 0, r, False, None
+        else:
+            return 5, r, False, None
 
     def getPropFromDict(self,rft,r,d,prop):
         if(prop in d):
@@ -853,7 +953,7 @@ class RfTransport():
 
 
 
-    def getPathBy(self,rft, r, coll):
+    def getPathBy(self,rft, r, coll, prop=None):
         if('Members'  not in coll):
             rft.printErr("Error: getPathBy: no members array in collection")
             return(None,1,None,False,None)
@@ -879,7 +979,18 @@ class RfTransport():
 
         elif(rft.oneOptn):
             if(numOfLinks > 1):
-                rft.printErr("Error: getPathBy --One option: more than one link in members array")
+                rc, r, j, d = rft.listCollection(rft, r, coll, prop)
+                id_list = []
+
+                if not rc and 'Members' in d:
+                    if prop is not None:
+                        if d['Members'] and '@odata.id' in d['Members'][0]:
+                            return(d['Members'][0]['@odata.id'],0,None,False,None)
+                    else:
+                        id_list = [m['Id'] for m in d['Members'] if 'Id' in m]
+                rft.printErr("Error: No target specified, but multiple {} IDs found: {}"
+                             .format(rft.subcommand, repr(id_list)))
+                rft.printErr("Re-issue command with '-I <Id>' to select target.")
                 return(None,1,None,False,None)
             if('@odata.id'  not in coll['Members'][0] ):
                 rft.printErr("Error: getPathBy --One option: improper formatted link-no @odata.id")
@@ -1021,9 +1132,9 @@ class RfTransport():
                     # create a member dict. Always include  Id and path
                     listMember={"Id": d["Id"], "@odata.id": d["@odata.id"] }
                     # if a property was specified to include, add it to the list dict
-                    if( prop is not None ):
+                    if( prop in d ):
                         listMember[prop]=propVal           
-                    # add the member to the listd
+                    # add the member to the list
                     members.append(listMember)
 
         #create base list dictionary
